import React from 'react';
import ReactDOM from 'react-dom';

var hot;
var passthrough = function (x) { return x; };
if (process.env.NODE_ENV !== 'production' && module.hot) {
    var app;
    var hmrRender;
    var container;
    function shouldReload() {
        return app && hmrRender && container;
    }
    
    hot = {
        patch: function patch(inst) {
            if (process.env.NODE_ENV !== 'production' && module.hot) {
                if (!isDvaInstance(inst)) {
                    throw Error('[hot.patch] app should be a `dva` instance');
                }
                var oldStart = inst.start;
                inst.start = (function (selector) {
                    if (isString(selector)) {
                        var maybeContainer = document.querySelector(selector);
                        if (isHTMLElement(maybeContainer)) {
                            console.log('[HMR] inited with dva-hot');
                            app = inst;
                            container = maybeContainer;
                            app.use({
                                onHmr: function onHmr(render) {
                                    hmrRender = render;
                                }
                            });
                        }
                    }
                    return oldStart.call(inst, selector);
                });
            }
            return inst;
        },
        model: function model(sourceModule) {
            if (process.env.NODE_ENV !== 'production' && sourceModule.hot) {
                return function (model) {
                    if (!isDvaModel(model)) {
                        throw Error('[hot.model] model shoule be a obj and has `namespace` prop');
                    }
                    var namespace = model.namespace;
                    sourceModule.hot.accept();
                    sourceModule.hot.dispose(function () {
                        if (namespace) {
                            app.unmodel(namespace);
                            namespace = undefined;
                        }
                    });
                    if (shouldReload()) {
                        try {
                            app.model(model);
                        } catch (e) {
                            console.error('error', e);
                        }
                    }
                    return model;
                };
            }
            return passthrough;
        },
        router: function router(sourceModule) {
            if (process.env.NODE_ENV !== 'production' && sourceModule.hot) {
                return function (router) {
                    if (!isFunction(router)) {
                        throw Error('[hot.router] router be a function');
                    }
                    sourceModule.hot.accept();
                    if (shouldReload()) {
                        var renderException = function (error) {
                            var RedBox = require('redbox-react');
                            ReactDOM.render(React.createElement(RedBox || RedBox.default, {
                                error: error
                            }), container);
                        };
                        var newRender = function (x) {
                            try {
                                hmrRender(x);
                            } catch (error) {
                                console.error('error', error);
                                renderException(error);
                            }
                        };
                        newRender(router);
                    }
                    return router;
                };
            }
            return passthrough;
        }
    };
    function isHTMLElement(node) {
        return typeof node === 'object' && node !== null && node.nodeType && node.nodeName;
    }
    
    function isString(str) {
        return typeof str === 'string';
    }
    
    function isFunction(func) {
        return typeof func === 'function';
    }
    
    function isDvaInstance(inst) {
        return inst && isFunction(inst.start);
    }
    
    function isDvaModel(obj) {
        return obj && obj.namespace;
    }
    
} else {
    var g = function () { return passthrough; };
    hot = {
        patch: passthrough,
        model: g,
        router: g
    };
}
var hot$1 = hot

export default hot$1;
//# sourceMappingURL=index.m.js.map
